//////////////////////////////////////////////////////////////////////////////
//
// $Id: logger.C,v 1.3 2001/07/21 10:27:12 assman Exp $
//
// $Log: logger.C,v $
// Revision 1.3  2001/07/21 10:27:12  assman
// Added semaphore interface
// Added functions to manage usec resolution time
// fixed bugfixes
// this is the first public release of the library
//
// Revision 1.2  2001/06/24 13:43:12  assman
// added full cvs tags
//
//
//////////////////////////////////////////////////////////////////////////////

#include <logger>

const int LOGGER::ERROR::success = 0;
const int LOGGER::ERROR::badlogfile = 1;
const int LOGGER::ERROR::badformat = 2;
const int LOGGER::ERROR::nullfmt = 3;

static const char *LOGGER::ERROR::errlist[] = {
  "Success",
  "Can't open file for logging",
  "Bad format tag",
  "Format tag is empty"
};

const int LOGGER::LEVEL::EMERG = 0;
const int LOGGER::LEVEL::ALERT = 1;
const int LOGGER::LEVEL::CRIT = 2;
const int LOGGER::LEVEL::ERR = 3;
const int LOGGER::LEVEL::WARNING = 4;
const int LOGGER::LEVEL::NOTICE = 5;
const int LOGGER::LEVEL::INFO  = 6;
const int LOGGER::LEVEL::DEBUG = 7;

const char *LOGGER::LEVEL::labels[] = {
  "EMERG",
  "ALERT",
  "CRIT",
  "ERR",
  "WARNING",
  "NOTICE",
  "INFO",
  "DEBUG" 
};

void Logger::log(int level, const char *fmt, ...) {

  //
  // only process logs less than the highest level
  // configured
  //

  if (level > _loglevel) return;

  va_list args;             // define argument list variable
  va_start (args,fmt);      // init list; point to last
 

  if (fmt == NULL) {
    _errnum = LOGGER::ERROR::nullfmt;
    return;
  }

  string ltpl = fmt;
  string p_str = "";
  int    p_int = 0;
  double  p_double = 0.0;
  char p_char = '\0';
  unsigned int f;
  unsigned int k;

  f = ltpl.find("%");

  while (f != string::npos) {
 
    if (f+1>= ltpl.size()) break;

    switch (ltpl[f+1]) {

    case 's': {
      p_str = (const char *)va_arg(args,char *);
      ltpl.replace(f,2,p_str);
       break;
    }
    case 'i':
    case 'd': {
      p_int = va_arg(args, int);
      ltpl.replace(f,2,TOOLS::itoa(p_int));
       break;
    }
    case 'f': {
      p_double = va_arg(args, double);
      ltpl.replace(f,2,TOOLS::ftoa(p_double));
      break;
     }
    case 'c': {
      p_char = va_arg(args, char);
      ltpl.replace(f,2,p_char);
      break;
     }
    }
  
    //
    // test for unsupported values
    // and bail out
    //

    k = ltpl.find("%");    
    if (f == k) {
      _errnum = LOGGER::ERROR::badformat;
      return;
    }
    else f = k;
  }

  va_end (args);  // clean up the system stack

  //
  // process the log
  //
  
  _process_log_entry(level,ltpl);
}

void ErrOutputLogger::_process_log_entry(int level, string str) {
    
  char datestr[80];
  bzero(datestr,80);
  struct tm *l_time;
  time_t l_time_t;
  string header;

  l_time_t = time(NULL);
  l_time = localtime(&l_time_t);
  strftime(datestr,80,"%b %d %H:%M:%S",l_time);

  header = string(datestr) + " " + _hostname + " " + 
    string(LOGGER::LEVEL::labels[level]) + " ";

  if (_ident != "") header = header + _ident;
  if (_pid) header = header + "(" + TOOLS::itoa(getpid()) + ")";
  if (_thr_id) header = header + "[" + TOOLS::itoa(pthread_self()) + "]";

  header = header + ": ";

  str = header + str;

  _display_log(str);
}

void ErrOutputLogger::_display_log(string str) {
  cerr << str << endl;
}

const bool ErrOutputLogger::logpid = true;
const bool ErrOutputLogger::logthreadpid = true;


Logger *LOGGER::Factory(string what, string servername="") {
	
	if (what == "STDERR") {
	    return (new ErrOutputLogger(servername,
					ErrOutputLogger::logpid,
					ErrOutputLogger::logthreadpid));
	}
	else {
	    if (what == "SYSLOG") {
		return(new SyslogLogger(servername.c_str()));
	    }
	    else {
		if (what != "") {
		    return(new FileLogger(what,servername,
					  ErrOutputLogger::logpid,
					  ErrOutputLogger::logthreadpid));
		}
		else {
		    return(new EmptyLogger());
		}
	    }
	}
	
	return(NULL);
	
    }
